from dataclasses import dataclass
from typing import Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class InputInputwrapper:
    classes: Optional[str] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class InputSuffix:
    text: Optional[str] = None
    html: Optional[str] = None
    classes: Optional[str] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class InputPrefix:
    text: Optional[str] = None
    html: Optional[str] = None
    classes: Optional[str] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class GovUKInput(govuk_frontend_base.GovUKComponent):
    """GOV.UK Input

    See: https://design-system.service.gov.uk/components/input/
    """

    id: str
    name: str
    type: Optional[str] = None
    inputmode: Optional[str] = None
    value: Optional[str] = None
    disabled: Optional[bool] = None
    describedBy: Optional[str] = None
    label: govuk_frontend_label.GovUKLabel
    hint: Optional[govuk_frontend_hint.GovUKHint] = None
    errorMessage: Optional[govuk_frontend_error_message.GovUKErrorMessage] = None
    prefix: Optional[InputPrefix] = None
    suffix: Optional[InputSuffix] = None
    formGroup: Optional[govuk_frontend_base.FormGroup] = None
    autocomplete: Optional[str] = None
    pattern: Optional[str] = None
    spellcheck: Optional[bool] = None
    autocapitalize: Optional[str] = None
    inputWrapper: Optional[InputInputwrapper] = None

    _jinja2_template = "govuk_frontend_jinja/components/input/macro.html"
    _macro_name = "govukInput"


COMPONENT = GovUKInput



from dataclasses import dataclass
from typing import Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class GovUKExitThisPage(govuk_frontend_base.GovUKComponent):
    """GOV.UK Exit This Page

    See: https://design-system.service.gov.uk/components/exit-this-page/
    """

    text: Optional[str] = None
    html: Optional[str] = None
    redirectUrl: Optional[str] = None
    id: Optional[str] = None
    activatedText: Optional[str] = None
    timedOutText: Optional[str] = None
    pressTwoMoreTimesText: Optional[str] = None
    pressOneMoreTimeText: Optional[str] = None

    _jinja2_template = "govuk_frontend_jinja/components/exit-this-page/macro.html"
    _macro_name = "govukExitThisPage"


COMPONENT = GovUKExitThisPage



from typing import List

from django import template

from govuk_frontend_django.components.header import GovUKHeader, HeaderNavigation
from govuk_frontend_django.components.tabs import TabsItems
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class HeaderNode(GovUKComponentNode):
    dataclass_cls = GovUKHeader

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["navigation"] = [
            dc
            for dc in self.get_sub_dataclasses_by_type(
                dataclass_cls=HeaderNavigation,
                many=True,
            )
        ]
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_header",
    node_cls=HeaderNode,
)


class HeaderNavItemNode(GovUKComponentNode):
    dataclass_cls = HeaderNavigation

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        href = component_kwargs["href"]
        component_kwargs["active"] = bool(href == context["request"].path)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents

        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_header_nav_item",
    node_cls=HeaderNavItemNode,
    end_if_not_contains=["text", "html"],
)



from typing import List

from django import template

from govuk_frontend_django.components.breadcrumbs import (
    BreadcrumbsItems,
    GovUKBreadcrumbs,
)
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class BreadcrumbsNode(GovUKComponentNode):
    dataclass_cls = GovUKBreadcrumbs

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        if "items" not in self.resolved_kwargs:
            component_kwargs["items"] = [
                node
                for node in self.get_sub_dataclasses_by_type(
                    dataclass_cls=BreadcrumbsItems,
                    many=True,
                )
            ]

        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_breadcrumbs",
    node_cls=BreadcrumbsNode,
    end_if_not_contains=["items"],
)


class BreadcrumbsItemsNode(GovUKComponentNode):
    dataclass_cls = BreadcrumbsItems


gds_register_tag(
    library=register,
    name="gds_breadcrumb_item",
    node_cls=BreadcrumbsItemsNode,
    has_end_tag=False,
)



from django import template

from govuk_frontend_django.components.base import (
    SummaryListRow,
    SummaryListRowsActions,
    SummaryListRowsActionsItem,
    SummaryListRowsKey,
    SummaryListRowsValue,
)
from govuk_frontend_django.components.summary_list import (
    GovUKSummaryList,
    SummaryListCard,
    SummaryListCardActions,
    SummaryListCardActionsItems,
    SummaryListCardTitle,
)
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class SummaryListCardActionsItemsNode(GovUKComponentNode):
    dataclass_cls = SummaryListCardActionsItems

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_card_actions_item",
    node_cls=SummaryListCardActionsItemsNode,
    end_if_not_contains=["text", "html"],
)


class SummaryListCardActionsNode(GovUKComponentNode):
    dataclass_cls = SummaryListCardActions

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["items"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=SummaryListCardActionsItems,
                many=True,
            )
        ]
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_card_actions",
    node_cls=SummaryListCardActionsNode,
)


class SummaryListCardTitleNode(GovUKComponentNode):
    dataclass_cls = SummaryListCardTitle

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_card_title",
    node_cls=SummaryListCardTitleNode,
    end_if_not_contains=["text", "html"],
)


class SummaryListCardNode(GovUKComponentNode):
    dataclass_cls = SummaryListCard

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["title"] = self.get_sub_dataclasses_by_type(
            dataclass_cls=SummaryListCardTitle,
            many=False,
        )
        component_kwargs["actions"] = self.get_sub_dataclasses_by_type(
            dataclass_cls=SummaryListCardActions,
            many=False,
        )
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_card",
    node_cls=SummaryListCardNode,
)


class SummaryListRowsValueNode(GovUKComponentNode):
    dataclass_cls = SummaryListRowsValue

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_row_value",
    node_cls=SummaryListRowsValueNode,
    end_if_not_contains=["text", "html"],
)


class SummaryListRowsActionsItemNode(GovUKComponentNode):
    dataclass_cls = SummaryListRowsActionsItem

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_row_actions_item",
    node_cls=SummaryListRowsActionsItemNode,
    end_if_not_contains=["text", "html"],
)


class SummaryListRowsActionsNode(GovUKComponentNode):
    dataclass_cls = SummaryListRowsActions

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["items"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=SummaryListRowsActionsItem,
                many=True,
            )
        ]
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_row_actions",
    node_cls=SummaryListRowsActionsNode,
)


class SummaryListRowsKeyNode(GovUKComponentNode):
    dataclass_cls = SummaryListRowsKey

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_row_key",
    node_cls=SummaryListRowsKeyNode,
    end_if_not_contains=["text", "html"],
)


class SummaryListRowNode(GovUKComponentNode):
    dataclass_cls = SummaryListRow

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["key"] = self.get_sub_dataclasses_by_type(
            dataclass_cls=SummaryListRowsKey,
            many=False,
        )

        component_kwargs["value"] = self.get_sub_dataclasses_by_type(
            dataclass_cls=SummaryListRowsValue,
            many=False,
        )

        component_kwargs["actions"] = self.get_sub_dataclasses_by_type(
            dataclass_cls=SummaryListRowsActions,
            many=False,
        )
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list_row",
    node_cls=SummaryListRowNode,
)


class SummaryListNode(GovUKComponentNode):
    dataclass_cls = GovUKSummaryList

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["card"] = self.get_sub_dataclasses_by_type(
            dataclass_cls=SummaryListCard,
            many=False,
        )

        component_kwargs["rows"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=SummaryListRow,
                many=True,
            )
        ]

        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_summary_list",
    node_cls=SummaryListNode,
)



from dataclasses import dataclass
from typing import List, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class FooterCopyright:
    text: Optional[str] = None
    html: Optional[str] = None


@dataclass(kw_only=True)
class FooterContentlicence:
    text: Optional[str] = None
    html: Optional[str] = None


@dataclass(kw_only=True)
class FooterNavigationItems:
    text: str
    href: str
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class FooterNavigation:
    title: str
    columns: Optional[int] = None
    width: Optional[str] = None
    items: Optional[List[FooterNavigationItems]] = None


@dataclass(kw_only=True)
class FooterMetaItems:
    text: str
    href: str
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class FooterMeta:
    visuallyHiddenTitle: Optional[str] = None
    html: Optional[str] = None
    text: Optional[str] = None
    items: Optional[List[FooterMetaItems]] = None


@dataclass(kw_only=True)
class GovUKFooter(govuk_frontend_base.GovUKComponent):
    """GOV.UK Footer

    See: https://design-system.service.gov.uk/components/footer/
    """

    meta: Optional[FooterMeta] = None
    navigation: Optional[List[FooterNavigation]] = None
    contentLicence: Optional[FooterContentlicence] = None
    copyright: Optional[FooterCopyright] = None
    containerClasses: Optional[str] = None

    _jinja2_template = "govuk_frontend_jinja/components/footer/macro.html"
    _macro_name = "govukFooter"


COMPONENT = GovUKFooter



from typing import List

from django import template

from govuk_frontend_django.components.base import TextAndHtml
from govuk_frontend_django.components.tabs import GovUKTabs, TabsItems
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class TabsNode(GovUKComponentNode):
    dataclass_cls = GovUKTabs

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["items"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=TabsItems,
                many=True,
            )
        ]
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_tabs",
    node_cls=TabsNode,
)


class TabsTabNode(GovUKComponentNode):
    dataclass_cls = TabsItems

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["panel"] = TextAndHtml(html=self.nodelist.render(context))
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_tabs_tab",
    node_cls=TabsTabNode,
)



import importlib
import pathlib
from dataclasses import is_dataclass
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Type, Union, cast

from django import template
from django.forms import BoundField
from django.template.base import (
    FilterExpression,
    Node,
    NodeList,
    Parser,
    Token,
    token_kwargs,
)
from django.template.context import Context
from django.template.defaulttags import ForNode, IfNode

from govuk_frontend_django.components.base import GovUKComponent

if TYPE_CHECKING:
    from _typeshed import DataclassInstance

register = template.Library()

SubDataclassWithNode = Tuple["DataclassInstance", Node]
DataclassDict = Dict[str, Any]
SubDataclassDictWithNode = Tuple[DataclassDict, Node]
SubDataclassDictWithOrWithoutNode = Union[DataclassDict, SubDataclassDictWithNode]


class ResolvingNode(Node):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.sub_dataclasses: List[SubDataclassWithNode] = []

    def clear(self) -> None:
        self.sub_dataclasses = []

    def resolve_nodelist(self, nodelist: NodeList, context: Context) -> None:
        for node in nodelist:
            if isinstance(node, GovUKComponentNode):
                dcls = node.resolve_dataclass(context, as_dict=False)
                assert is_dataclass(dcls)

                dcls_with_node = cast(SubDataclassWithNode, (dcls, node))
                self.sub_dataclasses.append(dcls_with_node)
                self.sub_dataclasses.extend(node.sub_dataclasses)
                continue

            if isinstance(node, IfNode):
                node = ComponentIfNode(conditions_nodelists=node.conditions_nodelists)

            if isinstance(node, ForNode):
                node = ComponentForNode(
                    loopvars=node.loopvars,
                    sequence=node.sequence,
                    is_reversed=node.is_reversed,
                    nodelist_loop=node.nodelist_loop,
                    nodelist_empty=node.nodelist_empty,
                )

            if hasattr(node, "resolve"):
                node.resolve(context)
            if hasattr(node, "sub_dataclasses"):
                self.sub_dataclasses.extend(node.sub_dataclasses)

    def resolve(
        self,
        context: Context,
    ) -> None:
        nodelist = getattr(self, "nodelist", NodeList())
        self.resolve_nodelist(nodelist, context)

    def get_sub_dataclasses_by_type(
        self,
        dataclass_cls: Type,
        many: bool = True,
        include_node: bool = False,
    ) -> Union[
        SubDataclassDictWithOrWithoutNode, List[SubDataclassDictWithOrWithoutNode]
    ]:
        if not hasattr(self, "sub_dataclasses"):
            return []

        sub_dataclasses: List[SubDataclassDictWithOrWithoutNode] = []
        for dc in self.sub_dataclasses:
            if isinstance(dc[0], dataclass_cls):
                if include_node:
                    sub_dataclasses.append(
                        (
                            dc[0].__dict__,
                            dc[1],
                        )
                    )
                else:
                    sub_dataclasses.append(dc[0].__dict__)

        if not many and sub_dataclasses:
            return sub_dataclasses[0]
        return sub_dataclasses


class ComponentIfNode(ResolvingNode, IfNode):
    """
    Custom IfNode that will only resolve the nodelist if the condition is True.
    """

    def resolve(self, context: Context, **kwargs):
        """
        See: IfNode.render
        """
        super().resolve(context)

        self.sub_dataclasses = []

        for condition, nodelist in self.conditions_nodelists:
            if condition is not None:  # if / elif clause
                try:
                    match = condition.eval(context.flatten())
                except template.VariableDoesNotExist:
                    match = None
            else:  # else clause
                match = True

            if match:
                self.resolve_nodelist(nodelist, context)


class ComponentForNode(ResolvingNode, ForNode):
    """
    Custom ForNode that will resolve the nodelist as per the loop.
    """

    def resolve(self, context: Context, **kwargs):
        """
        See: ForNode.render
        """
        assert isinstance(self.sequence, FilterExpression)

        self.nodelist = NodeList()

        if "forloop" in context:
            parentloop = context["forloop"]
        else:
            parentloop = {}
        with context.push():
            values = self.sequence.resolve(context, ignore_failures=True)
            if values is None:
                values = []
            if not hasattr(values, "__len__"):
                values = list(values)
            len_values = len(values)
            if len_values < 1:
                assert isinstance(self.nodelist_empty, NodeList)
                return self.nodelist_empty.render(context)
            if self.is_reversed:
                values = reversed(values)
            num_loopvars = len(self.loopvars)
            unpack = num_loopvars > 1
            # Create a forloop value in the context.  We'll update counters on each
            # iteration just below.
            loop_dict = context["forloop"] = {"parentloop": parentloop}
            for i, item in enumerate(values):
                # Shortcuts for current loop iteration number.
                loop_dict["counter0"] = i
                loop_dict["counter"] = i + 1
                # Reverse counter iteration numbers.
                loop_dict["revcounter"] = len_values - i
                loop_dict["revcounter0"] = len_values - i - 1
                # Boolean values designating first and last times through loop.
                loop_dict["first"] = i == 0
                loop_dict["last"] = i == len_values - 1

                pop_context = False
                if unpack:
                    # If there are multiple loop variables, unpack the item into
                    # them.
                    try:
                        len_item = len(item)
                    except TypeError:  # not an iterable
                        len_item = 1
                    # Check loop variable count before unpacking
                    if num_loopvars != len_item:
                        raise ValueError(
                            "Need {} values to unpack in for loop; got {}. ".format(
                                num_loopvars, len_item
                            ),
                        )
                    unpacked_vars = dict(zip(self.loopvars, item))
                    pop_context = True
                    context.update(unpacked_vars)
                else:
                    context[self.loopvars[0]] = item

                assert isinstance(self.nodelist_loop, NodeList)

                self.resolve_nodelist(self.nodelist_loop, context)

                if pop_context:
                    # Pop the loop variables pushed on to the context to avoid
                    # the context ending up in an inconsistent state when other
                    # tags (e.g., include and with) push data to context.
                    context.pop()

        return super().resolve(context)


class GovUKComponentNode(ResolvingNode):
    dataclass_cls: Type["DataclassInstance"]

    def __init__(
        self,
        extra_context: Dict[str, FilterExpression],
        nodelist: Optional[NodeList] = None,
        dataclass_cls: Optional[Type["DataclassInstance"]] = None,
    ):
        self.nodelist = nodelist or NodeList()
        self.extra_context = extra_context
        if dataclass_cls:
            self.dataclass_cls = dataclass_cls
        self.resolved_kwargs: Dict[str, Any] = {}
        self.resolved_dataclass = None
        super().__init__()

    def resolve(self, context: Context):
        super().resolve(context)

        self.resolved_kwargs = {
            key: val.resolve(context)  # type: ignore
            for key, val in self.extra_context.items()
        }

    def build_component_kwargs(self, context: Context):
        self.resolve(context)
        component_kwargs = self.resolved_kwargs.copy()
        return component_kwargs

    def resolve_dataclass(
        self, context: Context, as_dict=True
    ) -> Union["DataclassInstance", DataclassDict]:
        self.resolved_dataclass = self.dataclass_cls(  # type: ignore
            **self.build_component_kwargs(context)
        )
        if as_dict:
            return self.resolved_dataclass.__dict__
        return self.resolved_dataclass  # type: ignore

    def render(self, context: Context) -> str:
        super().render(context)
        resolved_dataclass = self.resolve_dataclass(context, as_dict=False)

        if hasattr(resolved_dataclass, "render"):
            assert isinstance(resolved_dataclass, GovUKComponent)
            return resolved_dataclass.render()
        return ""


FIELD_COMPONENTS = [
    "character-count",
    "checkboxes",
    "date-input",
    "fieldset",
    "file-upload",
    "input",
    "radios",
    "select",
    "textarea",
]
COMPLEX_COMPONENTS = [
    "breadcrumbs",
    "cookie-banner",
    "error-summary",
    "footer",
    "header",
    "pagination",
    "phase-banner",
    "summary-list",
    "table",
    "tabs",
]


@register.tag
def gds_component(parser: Parser, token: Token):
    bits = token.split_contents()
    component_name = bits[1].replace("'", "").replace('"', "")

    if component_name in FIELD_COMPONENTS:
        raise Exception(
            f"Use the gds_field_component tag for {component_name} components."
        )
    if component_name in COMPLEX_COMPONENTS:
        raise Exception(
            f"{component_name} is a complex component with it's own templatetag."
        )

    remaining_bits = bits[2:]
    extra_context = token_kwargs(remaining_bits, parser, support_legacy=True)

    # Component dataclass
    underscored_component_name = component_name.replace("-", "_")
    module_name = f"govuk_frontend_django.components.{underscored_component_name}"
    module = importlib.import_module(module_name)
    dataclass_cls = getattr(module, "COMPONENT")

    return GovUKComponentNode(
        nodelist=NodeList(),
        extra_context=extra_context,
        dataclass_cls=dataclass_cls,
    )


@register.simple_tag
def gds_field_component(field: BoundField, **kwargs):
    return "Not yet implemented"


class SetNode(Node):
    def __init__(self, nodelist: NodeList, asvar: str):
        self.nodelist = nodelist
        self.asvar = asvar

    def resolve(self, context: Context):
        context[self.asvar] = self.nodelist.render(context)

    def render(self, context: Context):
        self.resolve(context)
        return ""


@register.tag
def set(parser: Parser, token: Token):
    nodelist = parser.parse(("endset",))
    parser.delete_first_token()

    bits = token.split_contents()
    node = SetNode(nodelist=nodelist, asvar=bits[1])

    return node


@register.simple_tag
def gds_component_template(jinja2_template: str, macro_name: str, **kwargs):
    """Fallback template tag for rendering a GovUK component.

    Args:
        jinja2_template (str): The path to the Jinja2 template.
        macro_name (str): The name of the macro to render.
        **kwargs: The keyword arguments to pass to the macro.

    Usage:
        {% gds_component_template jinja2_template="govuk_frontend_jinja/components/warning-text/macro.html" macro_name="govukWarningText" text="WARNING: something is wrong!" %}
    """

    component = GovUKComponent()
    component.set_jinja2_template(jinja2_template)
    component.set_macro_name(macro_name)
    return component.render(component_data={**kwargs})


def gds_register_tag(
    library: template.Library,
    name: str,
    node_cls: Type[GovUKComponentNode],
    has_end_tag: bool = True,
    end_if_not_contains: Optional[List[str]] = None,
):
    end_if_not_contains = end_if_not_contains or []

    def template_tag(parser: Parser, token: Token):
        assert end_if_not_contains is not None

        bits = token.split_contents()
        remaining_bits = bits[1:]
        nodelist = template.NodeList()
        extra_context = token_kwargs(remaining_bits, parser, support_legacy=True)

        if has_end_tag:
            if not any(
                [
                    end_if_not_contain in extra_context
                    for end_if_not_contain in end_if_not_contains
                ]
            ):
                nodelist = parser.parse((f"end{name}",))
                parser.delete_first_token()

        return node_cls(
            extra_context=extra_context,
            nodelist=nodelist,
        )

    library.tag(name=name, compile_function=template_tag)
    library.tag(name=name + "_inline", compile_function=template_tag)


# Loop over python files in this directory and update the register object
# with the tags and filters defined in each file.
for path in pathlib.Path(__file__).parent.iterdir():
    if path.suffix == ".py" and path.name != "__init__.py":
        module_name = path.stem
        module = importlib.import_module(f".{module_name}", __package__)
        register.tags.update(module.register.tags)
        register.filters.update(module.register.filters)



from dataclasses import dataclass
from typing import List, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class GovUKAccordion(govuk_frontend_base.GovUKComponent):
    """GOV.UK Accordion

    See: https://design-system.service.gov.uk/components/accordion/
    """

    id: str
    headingLevel: Optional[int] = None
    rememberExpanded: Optional[bool] = None
    hideAllSectionsText: Optional[str] = None
    hideSectionText: Optional[str] = None
    hideSectionAriaLabelText: Optional[str] = None
    showAllSectionsText: Optional[str] = None
    showSectionText: Optional[str] = None
    showSectionAriaLabelText: Optional[str] = None
    items: List[govuk_frontend_base.AccordionItem]

    _jinja2_template = "govuk_frontend_jinja/components/accordion/macro.html"
    _macro_name = "govukAccordion"


COMPONENT = GovUKAccordion



from dataclasses import dataclass
from typing import List, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class CookieBannerMessagesActions:
    text: str
    type: Optional[str] = None
    href: Optional[str] = None
    name: Optional[str] = None
    value: Optional[str] = None
    classes: Optional[str] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class CookieBannerMessages:
    headingText: Optional[str] = None
    headingHtml: Optional[str] = None
    text: Optional[str] = None
    html: Optional[str] = None
    actions: Optional[List[CookieBannerMessagesActions]] = None
    hidden: Optional[bool] = None
    role: Optional[str] = None
    classes: Optional[str] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class GovUKCookieBanner(govuk_frontend_base.GovUKComponent):
    """GOV.UK Cookie Banner

    See: https://design-system.service.gov.uk/components/cookie-banner/
    """

    ariaLabel: Optional[str] = None
    hidden: Optional[bool] = None
    messages: List[CookieBannerMessages]

    _jinja2_template = "govuk_frontend_jinja/components/cookie-banner/macro.html"
    _macro_name = "govukCookieBanner"


COMPONENT = GovUKCookieBanner



from dataclasses import dataclass
from typing import List, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class HeaderNavigation:
    text: Optional[str] = None
    html: Optional[str] = None
    href: Optional[str] = None
    active: Optional[bool] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class GovUKHeader(govuk_frontend_base.GovUKComponent):
    """GOV.UK Header

    See: https://design-system.service.gov.uk/components/header/
    """

    homepageUrl: Optional[str] = None
    productName: Optional[str] = None
    serviceName: Optional[str] = None
    serviceUrl: Optional[str] = None
    navigation: Optional[List[HeaderNavigation]] = None
    navigationClasses: Optional[str] = None
    navigationLabel: Optional[str] = None
    menuButtonLabel: Optional[str] = None
    menuButtonText: Optional[str] = None
    containerClasses: Optional[str] = None
    useTudorCrown: Optional[bool] = None

    _jinja2_template = "govuk_frontend_jinja/components/header/macro.html"
    _macro_name = "govukHeader"


COMPONENT = GovUKHeader



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union

from django.utils.safestring import mark_safe
from jinja2 import (
    ChoiceLoader,
    Environment,
    PackageLoader,
    PrefixLoader,
    select_autoescape,
)

Attributes = Dict[str, Any]


class RenderableMixin:
    def get_context(self):
        raise NotImplementedError(
            "Subclasses of RenderableMixin must provide a get_context() method."
        )

    def render(self, template_name=None, context=None, renderer=None):
        raise NotImplementedError(
            "Subclasses of RenderableMixin must provide a render() method."
        )

    __str__ = render
    __html__ = render


@dataclass(kw_only=True)
class GovUKComponent(RenderableMixin):
    classes: Optional[str] = None
    attributes: Optional[Attributes] = None

    def set_jinja2_template(self, jinja2_template):
        self._jinja2_template = jinja2_template

    def set_macro_name(self, macro_name):
        self._macro_name = macro_name

    def build_jinja_template(self):
        return "".join(
            [
                "{%- from '",
                self._jinja2_template,
                "' import ",
                self._macro_name,
                " -%}",
                "{{ ",
                self._macro_name,
                "(data) }}",
            ]
        )

    def render(
        self,
        template_name=None,
        context=None,
        renderer=None,
        component_data=None,
    ):
        jinja_loader = ChoiceLoader(
            [
                PrefixLoader(
                    {"govuk_frontend_jinja": PackageLoader("govuk_frontend_jinja")}
                )
            ]
        )
        env = Environment(
            loader=jinja_loader,
            autoescape=select_autoescape(),
        )
        template = env.from_string(self.build_jinja_template())

        data = self.__dict__.copy()
        if component_data:
            data.update(component_data)

        def clean_data(data_to_clean: Union[Dict, List]):
            # Clean data
            # Iterate through the multi-level dict and remove any keys with a value of None
            if isinstance(data_to_clean, list):
                for item in data_to_clean:
                    clean_data(item)

            elif isinstance(data_to_clean, dict):
                for key, value in list(data_to_clean.items()):
                    if value is None:
                        del data_to_clean[key]
                    elif isinstance(value, dict) or isinstance(value, list):
                        clean_data(value)

        clean_data(data)

        return mark_safe(template.render(data=data))

    __str__ = render
    __html__ = render


@dataclass(kw_only=True)
class FieldsetLegend:
    text: str
    isPageHeading: bool
    classes: str


@dataclass(kw_only=True)
class TextAndHtml:
    text: Optional[str] = None
    html: Optional[str] = None


@dataclass(kw_only=True)
class FormGroup:
    classes: Optional[str] = None


"""
Accordion
"""


@dataclass(kw_only=True)
class AccordionItem:
    heading: Optional[TextAndHtml] = None
    summary: Optional[TextAndHtml] = None
    content: Optional[TextAndHtml] = None
    expanded: Optional[bool] = None


"""
Checkboxes
"""


@dataclass(kw_only=True)
class CheckboxesConditional:
    html: str


"""
Summary List
"""


@dataclass(kw_only=True)
class SummaryListRowsKey(TextAndHtml):
    classes: Optional[str] = None


@dataclass(kw_only=True)
class SummaryListRowsValue(TextAndHtml):
    classes: Optional[str] = None


@dataclass(kw_only=True)
class SummaryListRowsActionsItem:
    href: str
    text: Optional[str] = None
    html: Optional[str] = None
    visuallyHiddenText: Optional[str] = None
    classes: Optional[str] = None
    attributes: Optional[Attributes] = None


@dataclass(kw_only=True)
class SummaryListRowsActions:
    items: List[SummaryListRowsActionsItem]
    classes: Optional[str] = None


@dataclass(kw_only=True)
class SummaryListRow:
    classes: Optional[str] = None
    key: Optional[SummaryListRowsKey] = None
    value: Optional[SummaryListRowsValue] = None
    actions: Optional[SummaryListRowsActions] = None


SummaryListRows = List[SummaryListRow]



from dataclasses import dataclass
from typing import List, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class TableHead:
    text: Optional[str] = None
    html: Optional[str] = None
    format: Optional[str] = None
    classes: Optional[str] = None
    colspan: Optional[int] = None
    rowspan: Optional[int] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class TableRows:
    text: Optional[str] = None
    html: Optional[str] = None
    format: Optional[str] = None
    classes: Optional[str] = None
    colspan: Optional[int] = None
    rowspan: Optional[int] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class GovUKTable(govuk_frontend_base.GovUKComponent):
    """GOV.UK Table

    See: https://design-system.service.gov.uk/components/table/
    """

    rows: List[List[TableRows]]
    head: Optional[List[TableHead]] = None
    caption: Optional[str] = None
    captionClasses: Optional[str] = None
    firstCellIsHeader: Optional[bool] = None

    _jinja2_template = "govuk_frontend_jinja/components/table/macro.html"
    _macro_name = "govukTable"


COMPONENT = GovUKTable



from typing import List

from django import template

from govuk_frontend_django.components.accordion import GovUKAccordion
from govuk_frontend_django.components.base import AccordionItem, TextAndHtml
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class AccordionNode(GovUKComponentNode):
    dataclass_cls = GovUKAccordion

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["items"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=AccordionItem,
                many=True,
            )
        ]

        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_accordion",
    node_cls=AccordionNode,
)


class AccordionItemNode(GovUKComponentNode):
    dataclass_cls = AccordionItem

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["heading"] = TextAndHtml(text=component_kwargs["heading"])
        component_kwargs["summary"] = TextAndHtml(text=component_kwargs["summary"])
        component_kwargs["content"] = TextAndHtml(html=self.nodelist.render(context))

        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_accordion_item",
    node_cls=AccordionItemNode,
)



from dataclasses import dataclass
from typing import List, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class PaginationNext:
    text: Optional[str] = None
    html: Optional[str] = None
    labelText: Optional[str] = None
    href: str
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class PaginationPrevious:
    text: Optional[str] = None
    html: Optional[str] = None
    labelText: Optional[str] = None
    href: str
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class PaginationItems:
    number: Optional[str] = None
    visuallyHiddenText: Optional[str] = None
    href: str
    current: Optional[bool] = None
    ellipsis: Optional[bool] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class GovUKPagination(govuk_frontend_base.GovUKComponent):
    """GOV.UK Pagination

    See: https://design-system.service.gov.uk/components/pagination/
    """

    items: Optional[List[PaginationItems]] = None
    previous: Optional[PaginationPrevious] = None
    next: Optional[PaginationNext] = None
    landmarkLabel: Optional[str] = None

    _jinja2_template = "govuk_frontend_jinja/components/pagination/macro.html"
    _macro_name = "govukPagination"


COMPONENT = GovUKPagination



from typing import List, Optional

from django import template
from django.core.paginator import Page, Paginator
from django.utils.translation import gettext_lazy as _

from govuk_frontend_django.components.pagination import (
    GovUKPagination,
    PaginationItems,
    PaginationNext,
    PaginationPrevious,
)

register = template.Library()

# From Paginator.ELLIPSIS
ELLIPSIS = _("…")


@register.simple_tag
def gds_pagination(page_obj: Page):
    """GDS pagination template tag.

    Args:
        page_obj (Page):
            The Django Paginator Page object.

    Usage:
        This template tag is designed to be used with the Django pagination,
        below is an example of it being used with a ListView.

        *views.py:*
        ```python
        from django.views.generic import ListView
        from django.contrib.auth import get_user_model

        User = get_user_model()


        class UserListingView(ListView):
            template_name = "template.html"
            model = User
            paginate_by = 25
        ```

        *template.html:*
        ```django
        {% load govuk_frontend_django %}
        {% gds_pagination page_obj %}
        ```
    """

    previous: Optional[PaginationPrevious] = None
    next: Optional[PaginationNext] = None
    pagination_items: List[PaginationItems] = []

    if page_obj.has_previous():
        previous = PaginationPrevious(
            href=f"?page={page_obj.previous_page_number()}",
            labelText="Previous",
        ).__dict__  # type: ignore

    if page_obj.has_next():
        next = PaginationNext(
            href=f"?page={page_obj.next_page_number()}",
            labelText="Next",
        ).__dict__  # type: ignore

    for page_number in page_obj.paginator.get_elided_page_range(  # type: ignore
        page_obj.number, on_each_side=2, on_ends=1
    ):
        if page_number == ELLIPSIS:
            pagination_items.append(PaginationItems(href=None, ellipsis=True))
        else:
            pagination_items.append(
                PaginationItems(
                    number=page_number,
                    current=page_number == page_obj.number,
                    href=f"?page={page_number}",
                )
            )

    return GovUKPagination(previous=previous, next=next, items=pagination_items)



from django import template

from govuk_frontend_django.components.base import GovUKComponent
from govuk_frontend_django.components.phase_banner import GovUKPhaseBanner
from govuk_frontend_django.components.tag import GovUKTag
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class PhaseBannerNode(GovUKComponentNode):
    dataclass_cls = GovUKPhaseBanner

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        nodes_to_hide = []

        component_kwargs["tag"] = None
        component_kwargs["tag"], tag_node = self.get_sub_dataclasses_by_type(
            dataclass_cls=GovUKTag,
            many=False,
            include_node=True,
        )
        nodes_to_hide.append(tag_node)

        for node_to_hide in nodes_to_hide:
            self.nodelist.remove(node_to_hide)

        rendered_contents = self.nodelist.render(context).strip()

        for node_to_hide in nodes_to_hide:
            self.nodelist.append(node_to_hide)

        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_phase_banner",
    node_cls=PhaseBannerNode,
)



from dataclasses import is_dataclass
from typing import Any, Dict

from django import template
from django.forms import BoundField
from django.template.context import Context

from govuk_frontend_django.components.base import CheckboxesConditional
from govuk_frontend_django.components.checkboxes import CheckboxesItems, GovUKCheckboxes
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    DataclassDict,
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class FieldNode(GovUKComponentNode):
    def resolve(self, context: Context):
        super().resolve(context)
        self.bound_field: BoundField = self.resolved_kwargs["field"]
        self.resolved_kwargs["name"] = self.bound_field.name
        self.resolved_kwargs["fieldset"] = {
            "legend": {
                "text": "How would you like to be contacted?",
                "isPageHeading": True,
                "classes": "govuk-fieldset__legend--l",
            }
        }

    def build_component_kwargs(self, context: Context):
        component_kwargs = super().build_component_kwargs(context)
        component_kwargs.pop("field")
        return component_kwargs


class CheckboxesNode(FieldNode):
    dataclass_cls = GovUKCheckboxes

    def build_component_kwargs(self, context: Context):
        component_kwargs = super().build_component_kwargs(context)

        self.checkbox_conditional_items: Dict[Any, DataclassDict] = {}
        for checkbox_conditional_item, node in self.get_sub_dataclasses_by_type(  # type: ignore
            dataclass_cls=CheckboxesConditional,
            many=True,
            include_node=True,
        ):
            assert isinstance(node, GovUKComponentNode)
            assert isinstance(checkbox_conditional_item, Dict)

            conditional_value = node.resolved_kwargs["value"]
            self.checkbox_conditional_items[
                conditional_value
            ] = checkbox_conditional_item

        component_kwargs["items"] = []

        if self.bound_field.field.choices:
            for choice in self.bound_field.field.choices:
                item = CheckboxesItems(
                    value=choice[0],
                    text=choice[1],  # type: ignore
                )
                if item.value in self.checkbox_conditional_items:
                    item.conditional = self.checkbox_conditional_items[item.value]  # type: ignore
                component_kwargs["items"].append(item.__dict__)

        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_checkboxes",
    node_cls=CheckboxesNode,
)


class CheckboxConditionalNode(GovUKComponentNode):
    dataclass_cls = CheckboxesConditional

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents

        del component_kwargs["value"]
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_checkbox_conditional",
    node_cls=CheckboxConditionalNode,
    end_if_not_contains=["html"],
)



from dataclasses import dataclass
from typing import Any, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class CharacterCountCountmessage:
    classes: Optional[str] = None


@dataclass(kw_only=True)
class GovUKCharacterCount(govuk_frontend_base.GovUKComponent):
    """GOV.UK Character Count

    See: https://design-system.service.gov.uk/components/character-count/
    """

    id: str
    name: str
    rows: Optional[str] = None
    value: Optional[str] = None
    maxlength: str
    maxwords: str
    threshold: Optional[str] = None
    label: govuk_frontend_label.GovUKLabel
    hint: Optional[govuk_frontend_hint.GovUKHint] = None
    errorMessage: Optional[govuk_frontend_error_message.GovUKErrorMessage] = None
    formGroup: Optional[govuk_frontend_base.FormGroup] = None
    spellcheck: Optional[bool] = None
    countMessage: Optional[CharacterCountCountmessage] = None
    textareaDescriptionText: Optional[str] = None
    charactersUnderLimitText: Optional[Any] = None
    charactersAtLimitText: Optional[str] = None
    charactersOverLimitText: Optional[Any] = None
    wordsUnderLimitText: Optional[Any] = None
    wordsAtLimitText: Optional[str] = None
    wordsOverLimitText: Optional[Any] = None

    _jinja2_template = "govuk_frontend_jinja/components/character-count/macro.html"
    _macro_name = "govukCharacterCount"


COMPONENT = GovUKCharacterCount



from dataclasses import dataclass
from typing import List, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class SummaryListCardActionsItems:
    href: str
    text: Optional[str] = None
    html: Optional[str] = None
    visuallyHiddenText: Optional[str] = None
    classes: Optional[str] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class SummaryListCardActions:
    items: Optional[List[SummaryListCardActionsItems]] = None
    classes: Optional[str] = None


@dataclass(kw_only=True)
class SummaryListCardTitle:
    text: Optional[str] = None
    html: Optional[str] = None
    headingLevel: Optional[int] = None
    classes: Optional[str] = None


@dataclass(kw_only=True)
class SummaryListCard:
    title: Optional[SummaryListCardTitle] = None
    actions: Optional[SummaryListCardActions] = None
    classes: Optional[str] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class GovUKSummaryList(govuk_frontend_base.GovUKComponent):
    """GOV.UK Summary List

    See: https://design-system.service.gov.uk/components/summary-list/
    """

    rows: govuk_frontend_base.SummaryListRows
    card: Optional[SummaryListCard] = None

    _jinja2_template = "govuk_frontend_jinja/components/summary-list/macro.html"
    _macro_name = "govukSummaryList"


COMPONENT = GovUKSummaryList



from typing import List

from django import template

from govuk_frontend_django.components.footer import (
    FooterContentlicence,
    FooterCopyright,
    FooterMeta,
    FooterMetaItems,
    FooterNavigation,
    FooterNavigationItems,
    GovUKFooter,
)
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class FooterNode(GovUKComponentNode):
    dataclass_cls = GovUKFooter

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["navigation"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=FooterNavigation,
                many=True,
            )
        ]

        meta = self.get_sub_dataclasses_by_type(
            dataclass_cls=FooterMeta,
            many=False,
        )
        if not meta:
            meta = FooterMeta(items=[]).__dict__
        component_kwargs["meta"] = meta

        component_kwargs["contentLicence"] = self.get_sub_dataclasses_by_type(
            dataclass_cls=FooterContentlicence,
            many=False,
        )
        component_kwargs["copyright"] = self.get_sub_dataclasses_by_type(
            dataclass_cls=FooterCopyright,
            many=False,
        )
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_footer",
    node_cls=FooterNode,
)


class FooterNavigationNode(GovUKComponentNode):
    dataclass_cls = FooterNavigation

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["items"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=FooterNavigationItems,
                many=True,
            )
        ]
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_footer_nav",
    node_cls=FooterNavigationNode,
)


class FooterNavigationItemNode(GovUKComponentNode):
    dataclass_cls = FooterNavigationItems

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_footer_nav_item",
    node_cls=FooterNavigationItemNode,
    end_if_not_contains=["text", "html"],
)


class FooterMetaNode(GovUKComponentNode):
    dataclass_cls = FooterMeta

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["items"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=FooterMetaItems,
                many=True,
            )
        ]

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_footer_meta",
    node_cls=FooterMetaNode,
)


class FooterMetaItemNode(GovUKComponentNode):
    dataclass_cls = FooterMetaItems

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_footer_meta_item",
    node_cls=FooterMetaItemNode,
    end_if_not_contains=["text", "html"],
)


class FooterContentlicenceNode(GovUKComponentNode):
    dataclass_cls = FooterContentlicence

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_footer_content_licence",
    node_cls=FooterContentlicenceNode,
    end_if_not_contains=["text", "html"],
)


class FooterCopyrightNode(GovUKComponentNode):
    dataclass_cls = FooterCopyright

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in component_kwargs:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_footer_copyright",
    node_cls=FooterCopyrightNode,
    end_if_not_contains=["text", "html"],
)



from typing import Dict, Iterable, List, Optional, Tuple, Union

from django import template
from django.db.models import Model
from django.db.models.query import QuerySet

from govuk_frontend_django.components.table import GovUKTable, TableHead, TableRows

register = template.Library()


@register.simple_tag
def gds_table(columns: List[Tuple[str, str]], rows: QuerySet | List[Dict], **kwargs):
    """GDS table template tag.

    Args:
        columns (List[Tuple[str, str]]):
            A list of tuples, each tuple should contain the key and the human
            readable name.
        rows (Iterable):
            An iterable object, could be a list of Dicts or a Model Queryset

    Usage:
        ```django
        {% load govuk_frontend_django %}
        {% gds_table ... %}
        ```
    """

    if isinstance(rows, list):
        rows_as_list = rows
    else:
        fields = [column[0] for column in columns]
        rows_as_list = rows.values(*fields)  # type: ignore

    table_head: List[TableHead] = []
    for column in columns:
        table_head.append(TableHead(text=column[1]))

    table_rows: List[List[TableRows]] = []
    for row in rows_as_list:
        table_row: List[TableRows] = []
        for column in columns:
            table_row.append(TableRows(text=row[column[0]]).__dict__)  # type: ignore
        table_rows.append(table_row)

    return GovUKTable(head=table_head, rows=table_rows, **kwargs)



from typing import List

from django import template

from govuk_frontend_django.components.error_summary import (
    ErrorSummaryErrorlist,
    GovUKErrorSummary,
)
from govuk_frontend_django.templatetags.govuk_frontend_django import (
    GovUKComponentNode,
    gds_register_tag,
)

register = template.Library()


class ErrorSummaryNode(GovUKComponentNode):
    dataclass_cls = GovUKErrorSummary

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        component_kwargs["errorList"] = [
            node
            for node in self.get_sub_dataclasses_by_type(
                dataclass_cls=ErrorSummaryErrorlist,
                many=True,
            )
        ]
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_error_summary",
    node_cls=ErrorSummaryNode,
)


class ErrorSummaryErrorListItemNode(GovUKComponentNode):
    dataclass_cls = ErrorSummaryErrorlist

    def build_component_kwargs(self, context):
        component_kwargs = super().build_component_kwargs(context)

        rendered_contents = self.nodelist.render(context).strip()
        if rendered_contents and "html" not in self.extra_context:
            component_kwargs["html"] = rendered_contents
        self.clear()
        return component_kwargs


gds_register_tag(
    library=register,
    name="gds_error_summary_error_list_item",
    node_cls=ErrorSummaryErrorListItemNode,
    end_if_not_contains=["text", "html"],
)
